{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Import libraries"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "import ee\n",
    "import geemap"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Create an interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map = geemap.Map(center=[40, -100], zoom=4)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Add Earth Engine Python script"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "# Add Earth Engine dataset\n",
    "image = ee.Image(\"USGS/SRTMGL1_003\")\n",
    "\n",
    "#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n",
    "#  Chapter:      A2.3 Surface Water Mapping\n",
    "#  Checkpoint:   A23a\n",
    "#  Authors:      K. Markert, G. Donchyts, A. Haag\n",
    "#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n",
    "\n",
    "# *** Section 1 ***\n",
    "\n",
    "# Define a point in Cambodia to filter by location.\n",
    "point = ee.Geometry.Point(104.9632, 11.7686)\n",
    "\n",
    "Map.centerObject(point, 11)\n",
    "\n",
    "# Get the Sentinel-1 collection and filter by space/time.\n",
    "s1Collection = ee.ImageCollection('COPERNICUS/S1_GRD') \\\n",
    "    .filterBounds(point) \\\n",
    "    .filterDate('2019-10-05', '2019-10-06') \\\n",
    "    .filter(ee.Filter.eq('orbitProperties_pass', 'ASCENDING')) \\\n",
    "    .filter(ee.Filter.eq('instrumentMode', 'IW'))\n",
    "\n",
    "# Grab the first image in the collection.\n",
    "s1Image = s1Collection.first()\n",
    "\n",
    "# Add the Sentinel-1 image to the map.\n",
    "Map.addLayer(s1Image, {\n",
    "    'min': -25,\n",
    "    'max': 0,\n",
    "    'bands': 'VV'\n",
    "}, 'Sentinel-1 image')\n",
    "\n",
    "# Specify band to use for Otsu thresholding.\n",
    "band = 'VV'\n",
    "\n",
    "# Define a reducer to calculate a histogram of values.\n",
    "histogramReducer = ee.Reducer.histogram(255, 0.1)\n",
    "\n",
    "# Reduce all of the image values.\n",
    "globalHistogram = ee.Dictionary(\n",
    "    s1Image.select(band).reduceRegion({\n",
    "        'reducer': histogramReducer,\n",
    "        'geometry': s1Image.geometry(),\n",
    "        'scale': 90,\n",
    "        'maxPixels': 1e10\n",
    "    }).get(band)\n",
    ")\n",
    "\n",
    "# Extract out the histogram buckets and counts per bucket.\n",
    "x = ee.List(globalHistogram.get('bucketMeans'))\n",
    "y = ee.List(globalHistogram.get('histogram'))\n",
    "\n",
    "# Define a list of values to plot.\n",
    "dataCol = ee.Array.cat([x, y], 1).toList()\n",
    "\n",
    "# Define the header information for data.\n",
    "columnHeader = ee.List([\n",
    "    [\n",
    "    {\n",
    "        'label': 'Backscatter',\n",
    "        'role': 'domain',\n",
    "        'type': 'number'\n",
    "    },\n",
    "    {\n",
    "        'label': 'Values',\n",
    "        'role': 'data',\n",
    "        'type': 'number'\n",
    "    }, ]\n",
    "])\n",
    "\n",
    "# Concat the header and data for plotting.\n",
    "dataTable = columnHeader.cat(dataCol)\n",
    "\n",
    "# Create plot using the ui.Chart function with the dataTable.\n",
    "# Use 'evaluate' to transfer the server-side table to the client.\n",
    "# Define the chart and print it to the console.\n",
    "dataTable.evaluate(function(dataTableClient) {\n",
    "    chart = ui.Chart(dataTableClient) \\\n",
    "        .setChartType('AreaChart') \\\n",
    "        .setOptions({\n",
    "            'title': band + ' Global Histogram',\n",
    "            'hAxis': {\n",
    "                'title': 'Backscatter [dB]',\n",
    "                'viewWindow': {\n",
    "                    'min': -35,\n",
    "                    'max': 15\n",
    "                }\n",
    "            },\n",
    "            'vAxis': {\n",
    "                'title': 'Count'\n",
    "            }\n",
    "        })\n",
    "    print(chart)\n",
    "})\n",
    "\n",
    "# See:\n",
    "# https:#medium.com/google-earth/otsus-method-for-image-segmentation-f5c48f405e\n",
    "def otsu(histogram):\n",
    "    # Make sure histogram is an ee.Dictionary object.\n",
    "    histogram = ee.Dictionary(histogram)\n",
    "    # Extract relevant values into arrays.\n",
    "    counts = ee.Array(histogram.get('histogram'))\n",
    "    means = ee.Array(histogram.get('bucketMeans'))\n",
    "    # Calculate single statistics over arrays\n",
    "    size = means.length().get([0])\n",
    "    total = counts.reduce(ee.Reducer.sum(), [0]).get([0])\n",
    "    sum = means.multiply(counts).reduce(ee.Reducer.sum(), [0]) \\\n",
    "        .get([0])\n",
    "    mean = sum.divide(total)\n",
    "    # Compute between sum of squares, where each mean partitions the data.\n",
    "    indices = ee.List.sequence(1, size)\n",
    "\n",
    "def func_rhf(i):\n",
    "        aCounts = counts.slice(0, 0, i)\n",
    "        aCount = aCounts.reduce(ee.Reducer.sum(), [0]) \\\n",
    "            .get([0])\n",
    "        aMeans = means.slice(0, 0, i)\n",
    "        aMean = aMeans.multiply(aCounts) \\\n",
    "            .reduce(ee.Reducer.sum(), [0]).get([0]) \\\n",
    "            .divide(aCount)\n",
    "        bCount = total.subtract(aCount)\n",
    "        bMean = sum.subtract(aCount.multiply(aMean)) \\\n",
    "            .divide(bCount)\n",
    "        return aCount.multiply(aMean.subtract(mean).pow(2)) \\\n",
    "            .add(\n",
    "                bCount.multiply(bMean.subtract(mean).pow(2)))\n",
    "\n",
    "    bss = indices.map(func_rhf)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "    # Return the mean value corresponding to the maximum BSS.\n",
    "    return means.sort(bss).get([-1])\n",
    "\n",
    "\n",
    "# Apply otsu thresholding.\n",
    "globalThreshold = otsu(globalHistogram)\n",
    "print('Global threshold value:', globalThreshold)\n",
    "\n",
    "# Create list of empty strings that will be used for annotation.\n",
    "thresholdCol = ee.List.repeat('', x.length())\n",
    "# Find the index where the bucketMean equals the threshold.\n",
    "threshIndex = x.indexOf(globalThreshold)\n",
    "# Set the index to the annotation text.\n",
    "thresholdCol = thresholdCol.set(threshIndex, 'Otsu Threshold')\n",
    "\n",
    "# Redefine the column header information with annotation column.\n",
    "columnHeader = ee.List([\n",
    "    [\n",
    "    {\n",
    "        'label': 'Backscatter',\n",
    "        'role': 'domain',\n",
    "        'type': 'number'\n",
    "    },\n",
    "    {\n",
    "        'label': 'Values',\n",
    "        'role': 'data',\n",
    "        'type': 'number'\n",
    "    },\n",
    "    {\n",
    "        'label': 'Threshold',\n",
    "        'role': 'annotation',\n",
    "        'type': 'string'\n",
    "    }]\n",
    "])\n",
    "\n",
    "# Loop through the data rows and add the annotation column.\n",
    "i) {\n",
    "    i = ee.Number(i)\n",
    "    row = ee.List(dataCol.get(i))\n",
    "    return row.add(ee.String(thresholdCol.get(i)))\n",
    "})\n",
    "\n",
    "# Concat the header and data for plotting.\n",
    "dataTable = columnHeader.cat(dataCol)\n",
    "\n",
    "# Create plot using the ui.Chart function with the dataTable.\n",
    "# Use 'evaluate' to transfer the server-side table to the client.\n",
    "# Define the chart and print it to the console.\n",
    "dataTable.evaluate(function(dataTableClient) {\n",
    "    # loop through the client-side table and set empty strings to None\n",
    "    for i in range(0, dataTableClient.length, 1):\n",
    "        if (dataTableClient[i][2] === '') {\n",
    "            dataTableClient[i][2] = None\n",
    "        }\n",
    "\n",
    "    chart = ui.Chart(dataTableClient) \\\n",
    "        .setChartType('AreaChart') \\\n",
    "        .setOptions({\n",
    "            'title': band + \\\n",
    "                ' Global Histogram with Threshold annotation',\n",
    "            'hAxis': {\n",
    "                'title': 'Backscatter [dB]',\n",
    "                'viewWindow': {\n",
    "                    'min': -35,\n",
    "                    'max': 15\n",
    "                }\n",
    "            },\n",
    "            'vAxis': {\n",
    "                'title': 'Count'\n",
    "            },\n",
    "            'annotations': {\n",
    "                'style': 'line'\n",
    "            }\n",
    "        })\n",
    "    print(chart)\n",
    "})\n",
    "\n",
    "# Apply the threshold on the image to extract water.\n",
    "globalWater = s1Image.select(band).lt(globalThreshold)\n",
    "\n",
    "# Add the water image to the map and mask 0 (no-water) values.\n",
    "Map.addLayer(globalWater.selfMask(),\n",
    "    {\n",
    "        'palette': 'blue'\n",
    "    },\n",
    "    'Water (global threshold)')\n",
    "\n",
    "#  -----------------------------------------------------------------------\n",
    "#  CHECKPOINT\n",
    "#  -----------------------------------------------------------------------"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Display the interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map"
   ]
  }
 ],
 "metadata": {
  "anaconda-cloud": {},
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.6.1"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
